<?php

namespace Drupal\danse;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\user\UserDataInterface;

/**
 * Logger service to write messages and exceptions to an external service.
 */
class Service {

  use StringTranslationTrait;

  /**
   * @var \Drupal\danse\PluginManager
   */
  protected $pluginManager;

  /**
   * @var \Drupal\user\UserDataInterface
   */
  protected $userData;

  /**
   * @var \Drupal\danse\Query
   */
  protected $query;

  /**
   * @param \Drupal\danse\PluginManager $plugin_manager
   * @param \Drupal\user\UserDataInterface $user_data
   * @param \Drupal\danse\Query $query
   */
  public function __construct(PluginManager $plugin_manager, UserDataInterface $user_data, Query $query) {
    $this->pluginManager = $plugin_manager;
    $this->userData = $user_data;
    $this->query = $query;
  }

  /**
   * @return \Drupal\danse\PluginInterface[]
   */
  public function getPluginInstances(): array {
    $instances = [];
    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      try {
        $instances[$id] = $this->pluginManager->createInstance($id);
      }
      catch (PluginException $e) {
        // We can savely ignore this here.
      }
    }
    return $instances;
  }

  /**
   * @param array $form
   * @param array|string $callback
   */
  protected function addSubmitHandler(array &$form, $callback): void {
    $callback = is_string($callback) ? [$this, $callback] : $callback;
    if (isset($form['actions']['submit']['#submit'])) {
      foreach (array_keys($form['actions']) as $action) {
        if ($action !== 'preview'
          && isset($form['actions'][$action]['#type'])
          && $form['actions'][$action]['#type'] === 'submit') {
          $form['actions'][$action]['#submit'][] = $callback;
        }
      }
    }
    else {
      $form['#submit'][] = $callback;
    }
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function buildForm(array &$form, FormStateInterface $form_state): void {
    $submitHandler = [];
    foreach ($this->getPluginInstances() as $plugin) {
      if ($result = $plugin->buildForm($form, $form_state)) {
        $submitHandler[] = $result;
      }
    }
    foreach ($submitHandler as $item) {
      $this->addSubmitHandler($form, $item);
    }
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Session\AccountInterface $account
   * @param bool $collapsed
   */
  public function buildUserSubscriptionForm(array &$form, AccountInterface $account, $collapsed = TRUE): void {
    $roles = $account->getRoles();
    $checkboxes = [];
    foreach ($this->getPluginInstances() as $plugin) {
      foreach ($plugin->getSupportedSubscriptions($roles) as $key => $title) {
        $checkboxes[$key] = [
          '#type' => 'checkbox',
          '#title' => $title,
          '#default_value' => $this->userData->get('danse', $account->id(), $key),
        ];
      }
    }
    if ($collapsed && !empty($checkboxes)) {
      $form['danse'] = [
        '#type' => 'details',
        '#title' => $this->t('Subscribe to events'),
        '#open' => FALSE,
        '#tree' => TRUE,
        '#weight' => 9,
      ];
      /** @noinspection AdditionOperationOnArraysInspection */
      $form['danse'] += $checkboxes;
      $this->addSubmitHandler($form, 'submitUserSubscriptionForm');
    }
    else if (!$collapsed) {
      if (empty($checkboxes)) {
        $form['info'] = [
          '#markup' => $this->t('No events available for subscription.'),
        ];
      }
      else {
        $form['danse'] = [
          '#type' => 'container',
          '#tree' => TRUE,
        ];
        $form['danse'] += $checkboxes;
      }
    }
  }

  /**
   * Submit callback.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function submitUserSubscriptionForm(array &$form, FormStateInterface $form_state): void {
    $this->saveUserSubscriptionForm($form_state, $form_state->getFormObject()->getEntity());
  }

  /**
   * Save user settings.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   * @param \Drupal\Core\Session\AccountInterface $account
   */
  public function saveUserSubscriptionForm(FormStateInterface $form_state, AccountInterface $account) {
    foreach ($form_state->getValue('danse') as $key => $value) {
      $this->userData->set('danse', $account->id(), $key, (int) $value);
    }
  }

  /**
   * Create all outstanding notifications.
   */
  public function createNotifications(): void {
    foreach ($this->getPluginInstances() as $plugin) {
      $plugin->createNotifications();
    }
  }

  /**
   * @param \Drupal\danse\PayloadInterface $payload
   */
  public function markSeen(PayloadInterface $payload): void {
    foreach ($this->query->findNotificationsForCurrentUser($payload) as $notification) {
      $notification->markSeen();
    }
  }

}
